-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

--
-- Paths for the textures to display for calendar events
--
local calendarEventTypeTexture =
{
  [CALENDAR_EVENTTYPE_RAID] =
  {
    default  = "Interface\\LFGFrame\\LFGIcon-Raid",
    detailed = "Interface\\LFGFrame\\LFGIcon-"
  },
  [CALENDAR_EVENTTYPE_DUNGEON] =
  {
    default  = "Interface\\LFGFrame\\LFGIcon-Dungeon",
    detailed = "Interface\\LFGFrame\\LFGIcon-"
  },
  [CALENDAR_EVENTTYPE_PVP] =
  {
    default  = "Interface\\Calendar\\UI-Calendar-Event-PVP",
    detailed = ""
  },
  [CALENDAR_EVENTTYPE_MEETING] =
  {
    default = "Interface\\Calendar\\MeetingIcon",
    detailed = ""
  },
  [CALENDAR_EVENTTYPE_OTHER] =
  {
    default = "Interface\\Calendar\\UI-Calendar-Event-Other",
    detailed = ""
  }
};

--
-- Colours for calendar event titles
--
local calendarEventColor =
{
  [""]          = NORMAL_FONT_COLOR;
  ["CREATOR"]   = {r=0.54, g=0.75, b=1.0};
  ["MODERATOR"] = {r=0.54, g=0.75, b=1.0};
}

--
-- Text & colours for calendar event invite status
--
local calendarEventInviteInfo =
{
  ["UNKNOWN"] =
  {
    text  = UNKNOWN,
    color = NORMAL_FONT_COLOR
  },
  [CALENDAR_INVITESTATUS_CONFIRMED] =
  {
    text  = CALENDAR_STATUS_CONFIRMED,
    color = GREEN_FONT_COLOR
  },
  [CALENDAR_INVITESTATUS_ACCEPTED] =
  {
    text  = CALENDAR_STATUS_ACCEPTED,
    color = GREEN_FONT_COLOR
  },
  [CALENDAR_INVITESTATUS_DECLINED] =
  {
    text  = CALENDAR_STATUS_DECLINED,
    color = RED_FONT_COLOR
  },
  [CALENDAR_INVITESTATUS_OUT] =
  {
    text  = CALENDAR_STATUS_OUT,
    color = RED_FONT_COLOR
  },
  [CALENDAR_INVITESTATUS_STANDBY] =
  {
    text  = CALENDAR_STATUS_STANDBY,
    color = ORANGE_FONT_COLOR
  },
  [CALENDAR_INVITESTATUS_INVITED] =
  {
    text  = CALENDAR_STATUS_INVITED,
    color = NORMAL_FONT_COLOR
  },
  [CALENDAR_INVITESTATUS_SIGNEDUP] =
  {
    text  = CALENDAR_STATUS_SIGNEDUP,
    color = GREEN_FONT_COLOR
  },
  [CALENDAR_INVITESTATUS_NOT_SIGNEDUP] =
  {
    text  = CALENDAR_STATUS_NOT_SIGNEDUP,
    color = NORMAL_FONT_COLOR
  },
  [CALENDAR_INVITESTATUS_TENTATIVE] =
  {
    text  = CALENDAR_STATUS_TENTATIVE,
    color = ORANGE_FONT_COLOR
  }
};

local ownershipChildren =
{
  {name = "CalendarCreateEventFrame",   x = 75, y = 50},
  {name = "CalendarViewEventFrame",     x = 75, y = 50},
  {name = "CalendarTexturePickerFrame", x = 75, y = 50},
  {name = "CalendarContextMenu"                       }
};

--
-- Info for controlling ownership of the calendar event edit/view frames
--
local ownershipInfo = {};

--
-- List of all calendar player events, guild events and guild announcements
--
local calendarEvent = {};

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to get the list of calendar events
--
local function calendarEventsGetList()
  -- Local Variables
  local currentEvent;
  local eventCalType;
  local eventDate = {};
  local eventInviteStatus;
  local eventModStatus;
  local eventTexture;
  local eventTitle;
  local eventType;
  local monthOffset = 0;
  local numDays;
  local numEvents;

  -- Clear any existing calendar events
  wipe(calendarEvent);

  -- Get the current date
  _, eventDate.month, eventDate.day, eventDate.year = CalendarGetDate();

  -- Update the day displayed in the portrait
  NigglesCalendarEvents.day:SetText(eventDate.day);

  -- Get the number of days in the current month
  _, _, numDays = CalendarGetMonth();

  -- Get all the events from today for the next 28 days
  for dayIdx = 1, 28 do
    numEvents = CalendarGetNumDayEvents(monthOffset, eventDate.day);
    for eventIdx = 1, numEvents do
      eventTitle, eventDate.hour, eventDate.min, eventCalType, _, eventType,
      eventTexture, eventModStatus, eventInviteStatus =
        CalendarGetDayEvent(monthOffset, eventDate.day, eventIdx);
      if ((eventCalType == "GUILD_ANNOUNCEMENT") or
          (eventCalType == "GUILD_EVENT") or
          (eventCalType == "PLAYER")) then
        calendarEvent[#calendarEvent+1] = {};
        currentEvent = calendarEvent[#calendarEvent];
        currentEvent.index       = eventIdx;
        currentEvent.day         = eventDate.day;
        currentEvent.monthOffset = monthOffset;
        currentEvent.date        = date("%A, %Y-%m-%d, %H:%M", time(eventDate));
        currentEvent.title       = eventTitle;
        currentEvent.type        = eventType;
        currentEvent.texture     = eventTexture;
        currentEvent.modStatus   = eventModStatus;
        if (eventCalType ~= "GUILD_ANNOUNCEMENT") then
          currentEvent.inviteStatus = eventInviteStatus;
        else
          currentEvent.inviteStatus = 0;
        end
      end
    end

    -- Move on to the next day
    eventDate.day = eventDate.day+1;
    if (eventDate.day > numDays) then
      monthOffset   = monthOffset+1;
      eventDate.day = 1;
      eventDate.month, eventDate.year, numDays = CalendarGetMonth(monthOffset);
    end
  end

  -- Update the Calendar Events scroll frame
  NigglesCalendarEventsUpdate();

  return;
end

--
-- Function to work out what texture to display for an calendar event
--
local function calendarEventGetTexture(eventType, texture)
  -- Local Variables
  local texturePath =
    calendarEventTypeTexture[CALENDAR_EVENTTYPE_OTHER].default;

  -- Check if the event type has textures
  if (calendarEventTypeTexture[eventType] ~= nil) then
    -- Check if the event type has a detailed texture path
    if ((calendarEventTypeTexture[eventType].detailed ~= "") and
        (texture ~= nil) and
        (texture ~= "")) then
      texturePath = calendarEventTypeTexture[eventType].detailed..texture;
    elseif (calendarEventTypeTexture[eventType].default ~= "") then
      texturePath = calendarEventTypeTexture[eventType].default;
    end
  end

  return texturePath;
end

--
-- Function to check if the current calendar event is still in the future
--
local function calendarEventInFuture()
  -- Local Variables
  local currentDay;
  local currentHour;
  local currentMin;
  local currentMonth;
  local currentYear;
  local eventDay;
  local eventHour;
  local eventMin;
  local eventMonth;
  local eventYear;
  local inFuture = false;

  -- Get the calendar event's date & time
  _, _, _, _, _, _, _, _, eventMonth, eventDay, eventYear,
    eventHour, eventMin = CalendarGetEventInfo();
  if (eventYear ~= nil) then
    -- Get the current date & time
    currentHour, currentMin = GetGameTime();
    _, currentMonth, currentDay, currentYear = CalendarGetDate();

    -- Work out if the calendar event is still in the future
    if (eventYear > currentYear) then
      inFuture = true;
    elseif (eventYear == currentYear) then
      if (eventMonth > currentMonth) then
        inFuture = true;
      elseif (eventMonth == currentMonth) then
        if (eventDay > currentDay) then
          inFuture = true;
        elseif (eventDay == currentDay) then
          if (eventHour > currentHour) then
            inFuture = true;
          elseif (eventHour == currentHour) then
            if (eventMin > currentMin) then
              inFuture = true;
            end
          end
        end
      end
    end
  end

  return inFuture;
end

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to initialise the changes to the Calendar
--
function NigglesCalendarInit()
  -- Local Variables
  local numPoints;

  -- Add a hook to update the event frames as they are show
  hooksecurefunc("CalendarFrame_ShowEventFrame",
    NigglesCalenderInvitesShow);

  -- Add hooks to update the scroll frames as they are scrolled
  hooksecurefunc("CalendarCreateEventInviteListScrollFrame_Update",
    function()
      NigglesCalendarInvitesUpdate(
        CalendarCreateEventInviteListScrollFrame);
    end);
  hooksecurefunc("CalendarViewEventInviteListScrollFrame_Update",
    function()
      NigglesCalendarInvitesUpdate(
        CalendarViewEventInviteListScrollFrame);
    end);

  -- Hook into the Calendar frame
  CalendarFrame:HookScript("OnShow", NigglesCalendarEventsTakeOwnership);

  -- Initialise the ownership info for the Calendar children
  for _, owner in ipairs({CalendarFrame, NigglesCalendarEvents}) do
    ownershipInfo[owner] = {};
    for childIdx, childInfo in ipairs(ownershipChildren) do
      -- Initialise the child's info
      ownershipInfo[owner][childIdx]        = {};
      ownershipInfo[owner][childIdx].frame  = _G[childInfo.name];
      ownershipInfo[owner][childIdx].points = {};

      -- Initialise the child's anchor points
      if (owner == CalendarFrame) then
        numPoints = _G[childInfo.name]:GetNumPoints();
        for pointIdx = 1, numPoints do
          ownershipInfo[owner][childIdx].points[pointIdx] =
            {_G[childInfo.name]:GetPoint(pointIdx)};
        end
      elseif ((childInfo.x ~= nil) and (childInfo.y ~= nil)) then
        ownershipInfo[owner][childIdx].points[1] =
          {"TOPLEFT", owner, "TOPLEFT", childInfo.x, childInfo.y};
      end
    end
  end

  return;
end

--
-- Function to enable/disable 'Filter' button
--
function NigglesCalendarInvitesFilterEnable()
  -- Enable/Disable the Filter button
  if ((CalendarEventCanEdit()) and
      ((CalendarCreateEventFrame.mode == "create") or
       (calendarEventInFuture())) and
      ((NigglesDBC.settings.calEventFilter == false) or
       (next(Niggles.lastOnline) ~= nil))) then
    NigglesCalendarCreateEventFilterButton:Enable();
  else
    NigglesCalendarCreateEventFilterButton:Disable();
  end

  return;
end

--
-- Function called when a Calendar Invites scroll frame's parent is shown
--
function NigglesCalenderInvitesShow(frame)
  -- Local Variables
  local button;
  local buttons;
  local calendarType = CalendarEventGetCalendarType();
  local heading;
  local icon;
  local iconName;
  local numButtons;
  local scrollFrame;

  -- Work out which widgets to work with
  if (frame == CalendarCreateEventFrame) then
    heading     = CalendarCreateEventInviteListNameSortButton;
    scrollFrame = CalendarCreateEventInviteListScrollFrame;
  elseif (frame == CalendarViewEventFrame) then
    heading     = CalendarViewEventInviteListNameSortButton;
    scrollFrame = CalendarViewEventInviteListScrollFrame;
  end

  -- Show/Hide the number of invites
  if (heading ~= nil) then
    if (NigglesDBC.settings["CalEventsInvites"] == true) then
      heading:SetText(format("%s |cFFFFD000(%d)", NAME,
        CalendarEventGetNumInvites()));
    else
      heading:SetText(NAME);
    end
  end

  -- Check the scroll frame's buttons can be found and the icons don't exist
  if ((scrollFrame) and
      (type(scrollFrame.buttons) == "table") and
      (#scrollFrame.buttons > 0) and
      (not _G["Niggles"..scrollFrame.buttons[1]:GetName().."Online"])) then
    -- Create icons for each of the buttons in the scroll frame
    buttons    = scrollFrame.buttons;
    numButtons = #buttons;
    for buttonIdx = 1, numButtons do
      -- Assign info to more convenient variables
      button   = buttons[buttonIdx];
      iconName = "Niggles"..button:GetName().."Online";

      -- Create the icon, if required
      if (not _G[iconName]) then
        -- Create the icon as a texture
        icon = button:CreateTexture(iconName, "BORDER");
        icon:SetPoint("RIGHT", -60, 0);
        icon:SetWidth(14);
        icon:SetHeight(14);
        icon:SetTexture(READY_CHECK_WAITING_TEXTURE);
        icon:Show();
      end
    end
  end

  -- Update the scroll frame
  if (scrollFrame) then
    _G[scrollFrame:GetName().."_Update"]();
  end

  -- Make the Filter button visible, if required
  if (frame == CalendarCreateEventFrame) then
    -- Check if the Filter button has been created
    if (not NigglesCalendarCreateEventFilterButton) then
      -- Create the Filter button
      button = CreateFrame("Button", "NigglesCalendarCreateEventFilterButton",
        frame, "UIPanelButtonTemplate2");
      button:SetWidth(85);
      button:SetHeight(23);
      button:SetText(L["Filter"]);
      button:SetPoint("TOPRIGHT", CalendarCreateEventInviteList,
        "BOTTOMRIGHT", 0, -1);
      button:SetScript("OnClick", NigglesCalEventFilterInvites);
    end

    -- Set the state of the 'Filter' button
    if (NigglesDBC.settings["CalEventsFilter"] == true) then
      -- Adjust the size & positions of the standard Invite frames
      CalendarCreateEventInviteEdit:SetWidth(115);
      CalendarCreateEventInviteButton:SetPoint("TOPRIGHT",
        CalendarCreateEventInviteList, "BOTTOMRIGHT", -84, 1);

      -- Show the 'Filter' button
      NigglesCalendarCreateEventFilterButton:Show();
    else
      -- Restore the size & positions of the standard Invite frames
      CalendarCreateEventInviteEdit:SetWidth(192);
      CalendarCreateEventInviteButton:SetPoint("TOPRIGHT",
        CalendarCreateEventInviteList, "BOTTOMRIGHT", 0, 1);

      -- Hide the button
      NigglesCalendarCreateEventFilterButton:Hide();
    end

    -- Enable/Disable the Filter button
    NigglesCalendarInvitesFilterEnable();

    -- Show/Hide the Filter button
    if ((calendarType == "GUILD_EVENT") or
        (calendarType == "PLAYER")) then
      NigglesCalendarCreateEventFilterButton:Show();
    else
      NigglesCalendarCreateEventFilterButton:Hide();
    end
  end

  -- Force a guild roster update
  if (IsInGuild()) then
    GuildRoster();
  end

  return;
end

--
-- Function to update the 'Online' icon in a Calender Invites scroll frame
--
function NigglesCalendarInvitesUpdate(scrollFrame)
  -- Local Variables
  local button;
  local buttonInvitee;
  local buttonName;
  local buttons;
  local iconName;
  local iconTexture;
  local numButtons;

  -- Check the scroll frame's buttons can be found
  if ((type(scrollFrame.buttons) == "table") and
      (#scrollFrame.buttons > 0)) then
    -- Assign info to more convenient variables
    buttons = scrollFrame.buttons;
    numButtons = #scrollFrame.buttons;

    -- Make sure the player is in last online list, even if not in a guild
    Niggles.lastOnline[UnitName("PLAYER")] = 0;

    -- Update the icon for each button in the scroll frame
    for buttonIdx = 1, numButtons do
      -- Check if the button is shown
      button = buttons[buttonIdx];
      if (button:IsShown()) then
        -- Make sure the icon exists
        buttonName = button:GetName();
        iconName   = "Niggles"..buttonName.."Online";
        if (_G[iconName]) then
          if (NigglesDBC.settings.CalEventsOnline) then
            -- Work out what texture to display
            buttonInvitee = _G[buttonName.."Name"]:GetText();
            iconTexture   = READY_CHECK_WAITING_TEXTURE;
            if (Niggles.lastOnline[buttonInvitee]) then
              if (Niggles.lastOnline[buttonInvitee] == 0) then
                iconTexture = READY_CHECK_READY_TEXTURE;
              elseif (Niggles.lastOnline[buttonInvitee] > 0) then
                iconTexture = READY_CHECK_NOT_READY_TEXTURE;
              end
            end

            -- Set the icon's texture and make sure it's visible
            _G[iconName]:SetTexture(iconTexture);
            _G[iconName]:Show();
          else
            _G[iconName]:Hide();
          end
        end
      end
    end
  end

  return;
end

--
-- Function to filter the invites to the current calendar event
--
function NigglesCalEventFilterInvites()
  -- Local Variables
  local numInvites = CalendarEventGetNumInvites();
  local invitee;
  local idx;
  local modStatus;

  -- Filter each invitee in the event
  for idx = numInvites, 1, -1 do
    -- Get the name of the invitee
    invitee, _, _, _, _, modStatus = CalendarEventGetInvite(idx);

    -- Check if the invitee should be removed or made a moderator
    if (((NigglesDBC.settings.CalEventsInactive > 0) and
         (Niggles.lastOnline[invitee]) and
         (Niggles.lastOnline[invitee] >
           NigglesDBC.settings.CalEventsInactive*24)) or
        (NigglesTableContains(NigglesDB.nonRaiders, invitee))) then
      CalendarEventRemoveInvite(idx);
    elseif ((modStatus == "") and
            (NigglesTableContains(NigglesDB.moderators, invitee)) and
            (CalendarEventCanModerate(idx))) then
      CalendarEventSetModerator(idx);
    end
  end

  return;
end

--
-- Function to process updates to invites list for the current calendar event
--
function NigglesCalEventInviteListUpdate()
  if ((CalendarCreateEventFrame) and
      (CalendarCreateEventFrame:IsShown())) then
    CalendarCreateEventInviteListNameSortButton:SetText(
      format("%s |cFFFFD000(%d)", NAME, CalendarEventGetNumInvites()));
  elseif ((CalendarViewEventFrame) and
          (CalendarViewEventFrame:IsShown())) then
    CalendarViewEventInviteListNameSortButton:SetText(
      format("%s |cFFFFD000(%d)", NAME, CalendarEventGetNumInvites()));
  end

  return;
end

--
-- Function to enable/disable frames in the Calendar Events panel
--
function NigglesOptionsCalEventsEnableFrames()
  NigglesEnableFrame(NigglesOptionsCalEventsInactive,
    Niggles.updatedOptions.settings.CalEventsFilter);
  NigglesEnableFrame(NigglesOptionsCalEventsModerators,
    Niggles.updatedOptions.settings.CalEventsFilter);
  NigglesEnableFrame(NigglesOptionsCalEventsNonRaiders,
    Niggles.updatedOptions.settings.CalEventsFilter);

  return;
end

--
-- Function to show/hide the Calendar Events frame
--
function NigglesCalendarEventsOnClick(self, button)
  -- Check the 'Calendar Events' frame is enabled
  if (NigglesDBC.settings.CalEventsList == true) then
    --Show/Hide the Calendar and Calendar Events frames
    if (button == "LeftButton") then
      -- Hide the Calendar Events frame
      HideUIPanel(NigglesCalendarEvents);
    elseif (button == "RightButton") then
      -- Make sure the Calendar frame is hidden
      if ((CalendarFrame:IsVisible()) and
          (Calendar_Hide)) then
        Calendar_Hide();
      end

      -- Toggle the Calendar Events frame
      if (NigglesCalendarEvents:IsVisible()) then
        HideUIPanel(NigglesCalendarEvents);
      else
        ShowUIPanel(NigglesCalendarEvents);
      end
    end
  end

  return;
end

--
-- Function to initialise the Calender Events frame
--
function NigglesCalendarEventsOnLoad(frame)
  -- Local Variables
  local factionGroup = UnitFactionGroup("player");

  -- Work out which PvP logo to display
  if (factionGroup == "Alliance") then
    calendarEventTypeTexture[CALENDAR_EVENTTYPE_PVP].default =
      calendarEventTypeTexture[CALENDAR_EVENTTYPE_PVP].default.."02";
  else
    calendarEventTypeTexture[CALENDAR_EVENTTYPE_PVP].default =
      calendarEventTypeTexture[CALENDAR_EVENTTYPE_PVP].default.."01";
  end

  -- Set the attributes that make the frame a UI Panel
  frame:SetAttribute("UIPanelLayout-defined",   true);
  frame:SetAttribute("UIPanelLayout-enabled",   true);
  frame:SetAttribute("UIPanelLayout-area",      "left");
  frame:SetAttribute("UIPanelLayout-pushable",  2);
  frame:SetAttribute("UIPanelLayout-whileDead", true);

  -- Hide the frame's button bar
  ButtonFrameTemplate_HideButtonBar(frame);

  -- Set the frame's title
  frame.TitleText:SetText("Calendar Events");

  -- Set the frame's portrait
  frame.portrait:SetTexture(
     "Interface\\AddOns\\Niggles\\textures\\Portrait-Calendar");

  -- Adjust the size/position of the frame's inset
  frame.Inset:SetPoint("TOPLEFT", 4, -65);
  frame.Inset:SetPoint("BOTTOMRIGHT", -7, 6);

  -- Initialise the scroll frame
  frame.scrollFrame.update = NigglesCalendarEventsUpdate;
  HybridScrollFrame_CreateButtons(frame.scrollFrame,
    "NigglesCalendarEventButtonTemplate", 1, 0);
  frame.scrollFrame.scrollBar.doNotHide = true;

  -- Register for calendar update events
  frame:RegisterEvent("CALENDAR_NEW_EVENT");
  frame:RegisterEvent("CALENDAR_OPEN_EVENT");
  frame:RegisterEvent("CALENDAR_UPDATE_PENDING_INVITES");
  frame:RegisterEvent("CALENDAR_UPDATE_ERROR");
  frame:RegisterEvent("CALENDAR_UPDATE_EVENT_LIST");

  -- Get the list of events
  calendarEventsGetList();

  -- Hook into the Game Time frame
  GameTimeFrame:HookScript("OnClick", NigglesCalendarEventsOnClick);

  return;
end

--
-- Function to update the scroll frame in the Calendar Events frame
--
function NigglesCalendarEventsUpdate()
  -- Local Variables
  local button;
  local displayedHeight;
  local eventIdx;
  local totalHeight;
  local scrollFrame = NigglesCalendarEventsScrollFrame;
  local topOffset = HybridScrollFrame_GetOffset(scrollFrame);
  local numButtons = #scrollFrame.buttons;
  local numEvents = #calendarEvent;
  local textColor;
  local currentEvent;

  -- Make sure the top offset is valid
  if (topOffset > numEvents) then
    topOffset = numEvents;
    HybridScrollFrame_SetOffset(scrollFrame, topOffset);
  end

  -- Update the scroll frame's buttons
  for idx = 1, numButtons do
    button = scrollFrame.buttons[idx];
    eventIdx = topOffset+idx;
    if (eventIdx <= numEvents) then
      -- Assign the current event to a more convenient variable
      currentEvent = calendarEvent[eventIdx];

      -- Set the event's title
      button.title:SetText(currentEvent.title);
      textColor = calendarEventColor[currentEvent.modStatus];
      button.title:SetTextColor(textColor.r, textColor.g,
        textColor.b, textColor.a);

      -- Set the event's date/time
      button.date:SetText(currentEvent.date);

      -- Set the event's invite status
      if (calendarEventInviteInfo[currentEvent.inviteStatus] ~= nil) then
        button.inviteStatus:SetText(
          calendarEventInviteInfo[currentEvent.inviteStatus].text);
        textColor = calendarEventInviteInfo[currentEvent.inviteStatus].color;
        button.inviteStatus:SetTextColor(textColor.r, textColor.g,
          textColor.b, textColor.a);
        button.inviteStatus:Show();
      else
        button.inviteStatus:Hide();
      end

      -- Set the event's icon
      button.icon:SetTexture(calendarEventGetTexture(
        calendarEvent[eventIdx].type,
        calendarEvent[eventIdx].texture));

      -- Show/Hide the event's signup icon
      if (calendarEvent[eventIdx].inviteStatus == 1) then
        button.signup:Show();
      else
        button.signup:Hide();
      end

      -- Save the event the button is for
      button.event = currentEvent;

      -- Show the button
      button:Show();
    else
      -- Hide the button
      button:Hide();
    end
  end

  -- Update the scroll frame's height
  totalHeight = numEvents*scrollFrame.buttons[1]:GetHeight();
  displayedHeight = numButtons*scrollFrame.buttons[1]:GetHeight();
  HybridScrollFrame_Update(scrollFrame, totalHeight, displayedHeight);

  return;
end

--
-- Function to process events for the Calendar Events frame
--
function NigglesCalendarEventsOnEvent(frame, event, ...)
  -- Local Variables
  local calendarMonth;
  local calendarType;
  local calendarYear;
  local currentMonth;
  local currentYear;

  -- Check that the Calendar Events frame is visible
  if (NigglesCalendarEvents:IsVisible()) then
    -- Process the event
    if (event == "CALENDAR_OPEN_EVENT") then
      -- Check the calendar event type should be processed
      calendarType = ...;
      if ((calendarType == "PLAYER") or
          (calendarType == "GUILD_EVENT") or
          (calendarType == "GUILD_ANNOUNCEMENT")) then
        -- Check if the calendar event can be edited
        if (CalendarEventCanEdit()) then
          CalendarCreateEventFrame.mode = "edit";
          CalendarFrame_ShowEventFrame(CalendarCreateEventFrame);
        else
          CalendarFrame_ShowEventFrame(CalendarViewEventFrame);
        end
      end
    else
      -- Get the current date
      _, currentMonth, _, currentYear = CalendarGetDate();

      -- Get the month displayed by the calendar
      calendarMonth, calendarYear = CalendarGetMonth();

      -- Check the calendar is displaying the current month
      if ((calendarMonth == currentMonth) or
          (calendarYear  == currentYear)) then
        calendarEventsGetList();
      else
        CalendarSetAbsMonth(currentMonth, currentYear);
      end
    end
  end

  return;
end

--
-- Function to process events for the Calendar Events frame
--
function NigglesCalendarEventsOnShow(frame)
  -- Take ownership of the Calendar's children
  NigglesCalendarEventsTakeOwnership(frame);

  -- Update the list of events
  calendarEventsGetList();

  return;
end

--
-- Function to process clicks on Calendar Event buttons
--
function NigglesCalendarEventButtonOnClick(button)
  -- Check there is a calendar event associated with the button
  if (button.event ~= nil) then
    -- Open the calendar event
    CalendarOpenEvent(button.event.monthOffset, button.event.day,
      button.event.index);
  end

  return;
end

--
-- Function to take ownership of the frames used to edit/view events
--
function NigglesCalendarEventsTakeOwnership(frame)
  -- Give ownership of the Calendar's children to the frame
  if (ownershipInfo[frame] ~= nil) then
    for _, childInfo in ipairs(ownershipInfo[frame]) do
      childInfo.frame:SetParent(frame);
      childInfo.frame:ClearAllPoints();
      for _, point in ipairs(childInfo.points) do
        childInfo.frame:SetPoint(point[1], point[2], point[3],
          point[4], point[5]);
      end
    end
  end

  return;
end

--
-- Function to refresh values in the Niggles Options Calendar
-- Events panel
--
function NigglesOptionsRefreshCalEvents()
  -- Local Variables
  local widget;

  -- Initialise and update the Invites lists
  widget         = NigglesOptionsCalEventsModerators;
  widget.sibling = NigglesOptionsCalEventsNonRaiders;
  widget.list    = Niggles.updatedOptions.moderators;
  FauxScrollFrame_OnVerticalScroll(_G[widget:GetName().."Scrollbar"],
    0, 14, NigglesCharacterListUpdate);
  widget         = NigglesOptionsCalEventsNonRaiders;
  widget.sibling = NigglesOptionsCalEventsModerators;
  widget.list    = Niggles.updatedOptions.nonRaiders;
  FauxScrollFrame_OnVerticalScroll(_G[widget:GetName().."Scrollbar"],
    0, 14, NigglesCharacterListUpdate);

  -- Set the tooltips for some of the controls
  widget            = NigglesOptionsCalEventsInactiveEdit;
  widget.tooltip    = L["CalEventsInactiveTooltip"];
  widget.nglSetting = "CalEventsInactive";

  -- Initialise all the widgets to the correct state for the settings
  NigglesOptionsRefreshSettings(NigglesOptionsCalEvents);

  -- Initialise all the frames to the correct state for the settings
  NigglesOptionsCalEventsEnableFrames();

  return;
end
