-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local NUM_NODES_PER_PAGE = 18;

local flightNode     = {};
local flightList     = {};
local collapsedZones = {};

local nodeMapping =
{
  [L["Shattered Sun Staging Area"]] =
  {
    zone = L["Isle of Quel'Danas"],
    loc  = L["Shattered Sun Staging Area"]
  },
  [L["Dalaran"]] =
  {
    zone = L["Crystalsong Forest"],
    loc  = L["Dalaran"]
  },
  [L["Transitus Shield, Coldarra"]] =
  {
    zone = L["Borean Tundra"],
    loc  = L["Transitus Shield, Coldarra"]
  },
  [L["Hellfire Peninsula, The Dark Portal, Alliance"]] =
  {
    zone = L["Hellfire Peninsula"],
    loc  = L["The Dark Portal"]
  },
  [L["Hellfire Peninsula, The Dark Portal, Horde"]] =
  {
    zone = L["Hellfire Peninsula"],
    loc  = L["The Dark Portal"]
  },
  [L["Acherus: The Ebon Hold"]] =
  {
    zone = L["Eastern Plaguelands"],
    loc  = L["Acherus: The Ebon Hold"]
  },
  [L["The Exodar"]] =
  {
    zone = L["Azuremyst Isle"],
    loc  = L["The Exodar"]
  }
};

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to update the state of the 'All' button
--
local function buttonAllUpdateState()
  -- Local Variables
  local allCollapsed = true;
  local numRows = #(flightList);

  -- Check if all the zones in the list are collapsed
  for listIdx = 1, numRows do
    if ((tonumber(flightList[listIdx]) == nil) and
        (collapsedZones[flightList[listIdx]] == nil)) then
      allCollapsed = false;
      break;
    end
  end

  -- Update the button
  if (allCollapsed) then
    _G["NigglesFlightsAllButton"]:SetNormalTexture(
      "Interface\\Buttons\\UI-PlusButton-Up");
  else
    _G["NigglesFlightsAllButton"]:SetNormalTexture(
      "Interface\\Buttons\\UI-MinusButton-Up");
  end

  return;
end

--
-- Function to update the state of the button that toggles the displaying of
-- the 'Flights' panel
--
local function buttonToggleUpdateState()
  -- Local Variables
  local button = _G["NigglesFlightsToggleButton"];

  -- Set the texture of the button
  if (NigglesFlights:IsShown()) then
    button:SetNormalTexture(
      "Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up");
    button:SetPushedTexture(
      "Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down");
    button:SetDisabledTexture(
      "Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Disabled");
  else
    button:SetNormalTexture(
      "Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up");
    button:SetPushedTexture(
      "Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down");
    button:SetDisabledTexture(
      "Interface\\Buttons\\UI-SpellbookIcon-NextPage-Disabled");
  end

  return;
end

--
-- Function to create the list used in the scrolled frame. The list includes
-- elements for the zones and handles the hiding of locations if the zone is
-- collapsed
--
local function flightListCreate()
  -- Local Variables
  local currentZone = "";
  local isCollapsed = false;
  local numNodes = #(flightNode);

  -- Clear the list of flight nodes
  wipe(flightList);

  -- Create the list, including items for all zones and those locations that
  -- are in uncollapsed zones.
  for nodeIdx = 1, numNodes do
    -- Check if the current node is in a different zone
    if (flightNode[nodeIdx].zone ~= currentZone) then
      table.insert(flightList, flightNode[nodeIdx].zone);
      currentZone = flightNode[nodeIdx].zone;
      isCollapsed = collapsedZones[currentZone];
    end

    -- Add the location to the list, if its zone is not collapsed
    if (isCollapsed ~= true) then
      table.insert(flightList, nodeIdx);
    end
  end

  -- Update the state of the 'All' button
  buttonAllUpdateState();

  return;
end

--
-- Function to compare 2 flight nodes
--
local function flightNodeSort(first, second)
  -- Local Variables
  local retValue;

  -- Work out which order the nodes should be
  if (first.zone ~= second.zone) then
    retValue = (first.zone < second.zone);
  else
    retValue = (first.location < second.location);
  end

  return retValue;
end

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to update the Flights List in the Niggles Flights panel.
--
function NigglesFlightsListUpdate()
  -- Local Variables
  local button;
  local buttonText;
  local buttonHilite;
  local listIdx;
  local numRows = #(flightList);
  local scrollFrame = NigglesFlightsScrollFrame;
  local topLine = FauxScrollFrame_GetOffset(scrollFrame);
  local nodeIdx;

  -- Make sure the top line displayed is still valid
  if (topLine > numRows-NUM_NODES_PER_PAGE) then
    -- Work out the new top line
    topLine = numRows-NUM_NODES_PER_PAGE;
    if (topLine < 0) then
      topLine = 0;
    end

    -- Set the new top line
    FauxScrollFrame_SetOffset(scrollFrame, topLine);
  end

  -- Update all the buttons
  for buttonIdx = 1, NUM_NODES_PER_PAGE do
    -- Get the current button
    button       = _G["NigglesFlightsButton"..buttonIdx];
    buttonText   = _G["NigglesFlightsButton"..buttonIdx.."Text"];
    buttonHilite = _G["NigglesFlightsButton"..buttonIdx.."Highlight"];

    -- Work out the index of the flight node to display in the button
    listIdx = topLine+buttonIdx;
    if (listIdx <= numRows) then
      -- Update the button
      button:SetID(listIdx);
      if (tonumber(flightList[listIdx]) == nil) then
        buttonText:SetText(flightList[listIdx]);
        button:SetNormalFontObject(GameFontNormalLeft);
        button:Enable();
        if (collapsedZones[flightList[listIdx]] == true) then
          button:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
        else
          button:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
        end
        buttonHilite:SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight");
      else
        nodeIdx = flightList[listIdx];
        buttonText:SetText(" "..flightNode[nodeIdx].location);
        if (NigglesIsCity(flightNode[nodeIdx].location)) then
          button:SetNormalFontObject(GameFontNormalLeftGreen);
        else
          button:SetNormalFontObject(GameFontHighlight);
        end
        if (flightNode[nodeIdx].type == "REACHABLE") then
          button:Enable();
        else
          button:Disable();
        end
        button:SetNormalTexture("");
        buttonHilite:SetTexture("");
      end

      -- Show the button
      button:Show();
    else
      -- Update the button
      button:Disable();
      button:Hide();
    end
  end

  -- Update the scroll frame
  FauxScrollFrame_Update(scrollFrame, numRows, NUM_NODES_PER_PAGE, 16,
    nil, 0, 0, nil, 0, 0, true);

  return;
end

--
-- Function to update the Niggles Flights panel when it is shown
--
function NigglesFlightsOnShow()
  -- Local Variables
  local nodeType;
  local nodeName;
  local numNodes = NumTaxiNodes();
  local zoneName;
  local zoneLoc;

  -- Clear the flight nodes
  wipe(flightNode);

  -- Get the info for all the nodes
  for nodeIdx = 1, numNodes do
    nodeType = TaxiNodeGetType(nodeIdx);
    if (nodeType ~= "NONE") then
      -- Work out the zone and location to use for the node
      nodeName = TaxiNodeName(nodeIdx);
      if (nodeMapping[nodeName] ~= nil) then
        zoneName = nodeMapping[nodeName].zone;
        zoneLoc  = nodeMapping[nodeName].loc;
      else
        zoneLoc, zoneName = string.match(nodeName, "(.*[^,]+), ([^,]*)");
        if ((zoneLoc == nil) or (zoneName == nil)) then
          zoneName = nodeName;
          zoneLoc  = nodeName;
        end
      end

      -- Add the node to the table
      table.insert(flightNode,
        {
          index    = nodeIdx,
          zone     = L[zoneName],
          location = L[zoneLoc],
          type     = nodeType
        });
    end
  end

  -- Sort the table
  table.sort(flightNode, flightNodeSort);

  -- Create the flight list
  flightListCreate();

  -- Set the scroll frames to the top
  FauxScrollFrame_OnVerticalScroll(NigglesFlightsScrollFrame, 0, 16, nil);
  NigglesFlightsListUpdate();

  return;
end

--
-- Function to process a button click in the Flights list
--
function NigglesFlightsButtonOnClick(button)
  -- Local variables
  local listIdx = button:GetID();

  -- Take a flight to the flight node clicked
  if (listIdx > 0) then
    if (tonumber(flightList[listIdx]) == nil) then
      if (collapsedZones[flightList[listIdx]] == nil) then
        collapsedZones[flightList[listIdx]] = true;
      else
        collapsedZones[flightList[listIdx]] = nil;
      end
      flightListCreate();
      NigglesFlightsListUpdate();
    else
      TakeTaxiNode(flightNode[flightList[listIdx]].index);
      PlaySound("igMainMenuOptionCheckBoxOn");
    end
  end

  return;
end

--
-- Function to handle OnEnter events for a button in the Flights List
--
function NigglesFlightsButtonOnEnter(button)
  -- Local variables
  local listIdx = button:GetID();

  -- Make the Taxi Frame show the route
  if ((listIdx > 0) and (tonumber(flightList[listIdx]) ~= nil)) then
    TaxiNodeOnButtonEnter(
      _G["TaxiButton"..flightNode[flightList[listIdx]].index]);
  end

  return;
end

--
-- Function called when the Niggles Flights panel is hidden
--
function NigglesFlightsOnHide(frame)
  -- Update the state of the toggle button
  buttonToggleUpdateState();

  return;
end

--
-- Function called when the Niggles Flights panel is loaded
--
function NigglesFlightsOnLoad(frame)
  -- Local Variables
  local buttonName = frame:GetName().."Button";
  local button;

  -- Set the panel's title
  frame.title:SetText(L["Flight Destinations"]);

  -- Create the buttons for the Flights list
  for buttonIdx = 2, NUM_NODES_PER_PAGE do
    button = CreateFrame("Button", buttonName..buttonIdx,
      frame, "NigglesFlightsButtonTemplate");
    button:SetPoint("TOPLEFT", _G[buttonName..(buttonIdx-1)],
      "BOTTOMLEFT", 0, 0);
  end

  return;
end

--
-- Function to toggle the displaying of the Flights panel
--
function NigglesFlightsToggleOnClick(button)
  -- Toggle the displaying of the Flights panel
  if (NigglesFlights:IsShown()) then
    NigglesFlights:Hide();
  else
    NigglesFlights:Show();
  end
  PlaySound("igMainMenuOptionCheckBoxOn");

  -- Update the state of the button
  buttonToggleUpdateState();

  return;
end

--
-- Function to toggle all the zones
--
function NigglesFlightZonesToggle()
  -- Local Variables
  local allCollapsed = true;
  local numRows = #(flightList);

  -- Check if all the zones in the list are collapsed
  for listIdx = 1, numRows do
    if ((tonumber(flightList[listIdx]) == nil) and
        (collapsedZones[flightList[listIdx]] == nil)) then
      allCollapsed = false;
      break;
    end
  end

  -- Toggle the state of all the zones, depending on their current state
  if (allCollapsed) then
    -- Remove the zones in the list from the collapsed zones list
    for listIdx = 1, numRows do
      if (tonumber(flightList[listIdx]) == nil) then
        collapsedZones[flightList[listIdx]] = nil;
      end
    end
  else
    -- Add the zones in the list to the collapsed zones list
    for listIdx = 1, numRows do
      if (tonumber(flightList[listIdx]) == nil) then
        collapsedZones[flightList[listIdx]] = true;
      end
    end
  end

  -- Update the list
  flightListCreate();
  NigglesFlightsListUpdate();

  return;
end
